#ifndef CATMULL_H
#define CATMULL_H

#include "glslprogram.h"
#include "point.h"
#include "lab_color.h"
#include "global_param.h"
#include <QVector>
#include <QColor>
#include <QImage>
#define _USE_MATH_DEFINES
#include <math.h>

class Catmull  
{
public:
	static void interpolate(Catmull* c1, Catmull* c2, Catmull* cResult, double t);//interpolate between c1 and c2: cResult = t*c2 + (1-t)*c1, 0<=t<=1
	//precondition: c1 and c2 have the same number of control points
	//draw the curve;
	//if tags = true, draw the curve with name tages attached to each piece
	void draw(bool tags=false);
	//draw the curve using the global IDs as tags; used for inserting/erasing attribute control points at global positions
	void drawWithGlobalIDTags();
	void drawNormalX();//draw the curve with the X derivative color
	void drawNormalY();//draw the curve with the Y derivative color
	void drawColorConstraintsTriangleStrip(double zoom);//draw the color constraints on each side
        void drawColorConstraints(double zoom);//draw the color constraints on each side
	void drawBlurAmount();//draw the amount of blur per pixel
	void drawOverlap(double zoom);//draw in the stencil buffer to detect overlapping areas
	void drawControlPoints();//draw the spline control points
	void drawAttribControlPoints(bool tags=false);//draw the color and blur control points
	void drawBoundingBox(); //draw the bounding nox

	void setClosedCurve(bool _closedCurve);
	bool isClosedCurve();

	//GEOMETRY CONTROL POINTS
	void addPoint(double x,double y);//add the (x,y) point in the control point list, read the left or right color in the reference buffer if refL or refR is true
	void addPolLinePoint(double x, double y);
	void insertPoint(double x,double y, int id, double t);//insert at the proper place the (x,y) point in the control point list
	void erasePoint(int id);//erase the control point at id pos from the control point list
        //split a curve, if this is a closed curve, return false,
        //else: return true and init c2 with the remaining curve: c -> c + c2
        bool split(double x,double y, int id, double t, Catmull* c2);


	//ATTRIBUTE CONTROL POINTS

	//adds a color; computes the ID 
	void addLeftColorPoint(bool refL=false, double x=0,double y=0);
	void addRightColorPoint(bool refR=false, double x=0,double y=0);

	void addLeftColorPoint(int colorID, bool useDefault=true, QColor lC = QColor(0,0,0));//use the default color
	void addRightColorPoint(int colorID, bool useDefault=true, QColor rC = QColor(0,0,0));//use the default color
	void addBlurPoint(int blurID, bool useDefault, double sigmaBlur=0);//use the default blur
        void sampleColors(AttribType attribTYPE);//sample colors in the reference image (attribTYPE specify wich side)
		void sampleFromExample();//sample colors in the example image

	//insert a control point at a position given by globalID in the attribType control points list
	void insertAttribPoint(int globalID, AttribType attribType);
        //insert a control point at a position given by globalID in the attribType control points list
        //and return its position in the attribType control points list
        int insertAttribPointPos(int globalID, AttribType attribTYPE);
	//erase the control point closest to globalID in the attribType control points list
	void eraseAttribPoint(int globalID, AttribType attribType);

	void setSigma(int sigma, int id);//set the sigma at control point having the id "id" 
	int getSigma(int id);//get the sigma at control point having the id "id"


	void increaseBlur();
	void decreaseBlur();
	//set the current color and blur; used by default
	void setBlur(int s);
	void setLeftColor(QColor C);
	void setRightColor(QColor C);
	void setLifetime(int l);

	int getBlur();//get the current color and blur
	QColor getLeftColor();
	QColor getRightColor();
	int getLifetime();

	//change attributes globally
	void setGlobalBlur(int shift_sigma);
	//smooth the spline
	void smooth();
        //contrast
        void increaseContrast();
        void decreaseContrast();

	//set the current moving point color
	void setControlPointLeftColor(QColor C, int id);
	void setControlPointRightColor(QColor C, int id);

        //switch left and right colors
        void switchColors();

        //activate or deactivate the stylization
        void stylize(){bDrawStraight=!bDrawStraight;};

        //init the example image and gradient, pixelSize, reference size, reference buffer id and shaders. Must be called once OpenGl is all set.
	static void initStaticDatas(double pSize, int wR, int hR, GLuint idR, QImage* img, QImage* grad);
	static void zoomIn();//change the controlPointSize according to the zoom
	static void zoomOut();
	static void setLifetimeThreshold(int th);
	void setMovingPoint(int id);//set the control point i to the be current moving point
	void releaseMovingPoint();//release the current moving point
	bool movePoint(double x,double y);//move the current moving point to (x,y)
	int getMovingPoint(){return movingPoint;};
	int getNbPoint(){return nbPoint;};
	int getNbLeftColors(){return nbLeftColors;};
	int getNbRightColors(){return nbRightColors;};
	int getNbBlurPoints(){return nbSigmas;};
	Point getCtrlPoint(int p){return controlPoints[p];};
	QColor getLeftColor(int p){return leftColors[p];};
	QColor getRightColor(int p){return rightColors[p];};
	int getBlur(int p){return sigmas[p];};
        int getMeanBlur();
        void pasteBlur(int b);//paste a uniform blur on the spline

	int getLeftColorID(int p){return leftColorsIDs[p];};
	int getRightColorID(int p){return rightColorsIDs[p];};
	int getBlurID(int p){return sigmasIDs[p];};
	//return: true if a point is currently moving 
	void getBoundingBox(Point& leftUp, Point& rightDown);
        QVector<Point> getControlPoints(){return controlPoints;};

	//dragging		
	void setDragPoint(double x, double y);//initiate dragging a selected spline
	void dragSpline(double x, double y);//drag a selected spline

        //snake
        void iterSnake();

	//set selected param
	bool isSelected();
	void setSelected(bool selected);

	void setUpDefaultParam();
	Catmull();
	Catmull(QColor leftColor, QColor rightColor, int s);
	~Catmull();

	//public static member
	static int NB_STEPS;

/////////////////////////////////////////////////////////////////////////////////////////////////////
	QVector<Point> getPointPosition(){return pointPosition;};
	QVector<Point> getPointNormal(){return pointNormal;};
	QVector<QColor> getPointColor(){return pointColor;};
	QVector<int> getPointBlur(){return pointBlur;};
	int getNbConstraint(){return pointPosition.size();};
/////////////////////////////////////////////////////////////////////////////////////////////////////
////






private:
	//PRIVATE FUNCTIONS
	Point ComputeP(double t, int start);//compute P(t)
	Point ComputePNormal(double t, int startPoint);//compute the normal at P(t)
        Point ComputePTangent(double t, int startPoint);//compute the tangent at P(t)
	double ComputeGr(double left_red, double right_red);
	double ComputeGg(double left_green, double right_green);
	double ComputeGb(double left_blue, double right_blue);
	QColor ComputeCLeft(double t, int start);//compute left color Cleft(t)
	QColor ComputeCRight(double t, int start);//compute right color Cright(t)
	int ComputeBlur(double t, int start);//compute the blur sigma(t)

	//returns a value between 0 and 1, used to interpolate color between color control points
        //and update the attribute index if necessary
	double ComputeAttribT(double geomT, int geomINDEX, int& attribINDEX, AttribType attribTYPE);
        //returns a value between 0 and 1, used to interpolate color between color control points
        //and compute the attribute index
        double ComputeAttribTAttribIndex(double geomT, int geomINDEX, int& attribINDEX, AttribType attribTYPE);
	//returns a value between 0 and 1, and fills in geomINDEX
	double ComputeGeomT(int globalID, int& geomINDEX);
	//update the IDs whn a geometry control point is deleted or inserted (so the total nb of steps changes)
	void updateAttribIDs(int controlPointINDEX, bool isBeingAdded);
	//update bounding box
	void updateBoundingBox();
	//vectorizes a set of colors; 
	QVector<int> vectorizedColorSet(QVector<QColor> colors, QVector<int> colorsIDs, double maxDistance);
	int computeBiggestError(QVector<int>& colorPolLine, QVector<QColor> colors, QVector<int> colorsIDs, double& biggestError);
	double labColorDist(LabColor& color1, LabColor& color2);
	LabColor RGBtoLab( QColor rgb);
	QColor computeColorLinearInterpolation(double t, QColor& color1, QColor& color2);



	//***********************************
	//PRIVATE MEMBERS

	//static parameters
	static double pas;
	static int lineWidth;//set thickness of lines
	static double controlPointsSize;//in fact, the width of the square marking the control point
	static double boundingBoxPadding;//by how much we augment the bounding box
	static int lifetimeThreshold;
	//blur
	static double pixelSize;
	//reference buffer
	static int wRef;
	static int hRef;
	static GLuint refBuffer;
        //example image
        static QImage* exampleImg;
        static QImage* exampleGradient;//gradient norm of the example image

	//shader
	static GLSLProgram colorConstraintsProgram;

        //stylization: straight lines
        bool bDrawStraight;

	//------------------------
	int nbPoint; //nb control points
	int nbLeftColors; //nb left colors control points
	int nbRightColors;//nb left colors control points
	int nbSigmas;//nb blur control points
	int lifetime;

	QVector<Point> controlPoints;//list of the control points
	QVector<Point> polLinePoints;//list of points on the polyline
	QVector<QColor> leftColors;//list of the colors on left color control points
	QVector<QColor> rightColors;//list of the colors on right color control points
	QVector<int> sigmas;//list of the blur value at blur control points

	//IDs
	//we consider there're 10 steps for each bezier; (NB_STEPS)
	//in total, steps go from 0 to 10*nbBeziers
	//IDs indicate at what step the color, blur or control point appear
	//for geometry control points, we don't need to store the IDs
	QVector<int> leftColorsIDs;//list left color control points IDs
	QVector<int> rightColorsIDs;//list right color control points IDs
	QVector<int> sigmasIDs;//list blur control points IDs

	//------------------------
	//current color and blur
	QColor leftColor;
	QColor rightColor;
	int sigma;
	int movingPoint;//id of the current moving point. -1 if no moving point.

	//the bounding box left-up, right-down corners;
	Point leftUp;
	Point rightDown;

	//drag data
	Point dragPoint;
	//the curve is closed or not
	bool closedCurve;
	//is selected or not
	bool selectedSpline;

	
	//nos variables pour notre solveur-----------------------------------------------------------------------------------
	//
	//
	//
	QVector<Point> pointPosition;
	QVector<QColor> pointColor;
	QVector<Point> pointNormal;
	QVector<int> pointBlur;
};

#endif 

