#include "interface.h"
#include <QColorDialog>
#include <QStyle>
#include <QPainter>
#include <QStyleOption>

#include "iostream"


Interface::Interface(QWidget *parent)
: QMainWindow(parent)
{
	setupUi(this);

	//I cannot create  aQButtonGroup in QtDesigner, so I'm creating it here
	//all manipMode children should be push buttons
	manipModeGroup = new QButtonGroup(this);
	QList<QPushButton*>  buttonList = manipMode->findChildren<QPushButton *>();
	QList<QPushButton*>::iterator it;
	for(it=buttonList.begin(); it!=buttonList.end();it++)
	{
		manipModeGroup->addButton(*it);
	}

	leftColor = QColor(leftColorInit[0],leftColorInit[1],leftColorInit[2]);
	rightColor = QColor(rightColorInit[0],rightColorInit[1],rightColorInit[2]);
        bucketColor = QColor(bucketColorInit[0],bucketColorInit[1],bucketColorInit[2]);
	sigma = 0;
        nbIter=3;

        QPixmap pxmapColor(10,10);
        pxmapColor.fill(leftColor);
        QIcon icL(pxmapColor);
	buttonLeftColor->setIcon(icL);
        pxmapColor.fill(rightColor);
        QIcon icR(pxmapColor);
        buttonRightColor->setIcon(icR);
        QPixmap pxmapBucket(12,25);
        pxmapBucket.fill(bucketColor);
        QIcon icB(pxmapBucket);
        buttonBucketColor->setIcon(icB);

	sliderBlur->setValue((int)sigma);

	//Style to mark that a button is pressed or not;
	stylePressed = QString("QPushButton { background-color: rgb(200,200,200); border-style: outset; border-width: 0.5px; border-radius: 3px; border-color: rgb(0,0,0);}");
	styleUnpressed = QString("QPushButton { background-color: rgb(255,255,255); border-style: outset; border-width: 0.5px; border-radius: 3px; border-color: rgb(0,0,0);}");
	styleDefault = buttonLeftColor->style();

	connect(buttonLeftColor,SIGNAL(clicked()),this,SLOT(changeLeftColor()));
	connect(buttonRightColor,SIGNAL(clicked()),this,SLOT(changeRightColor()));
	connect(sliderBlur,SIGNAL(valueChanged(int)),this,SLOT(changeBlur(int)));
        connect(sliderIter,SIGNAL(valueChanged(int)),this,SLOT(changeIter(int)));
        connect(sliderLifetimeThreshold,SIGNAL(valueChanged(int)),this,SLOT(changeLifetimeThreshold(int)));
	connect(checkBoxShowSplines,SIGNAL(stateChanged(int)),this,SLOT(showSplines(int)));
	connect(checkBoxShowExampleImg,SIGNAL(stateChanged(int)),this,SLOT(showExampleImg(int)));
        connect(checkBoxActivateBlur,SIGNAL(stateChanged(int)),this,SLOT(activateBlur(int)));
        connect(checkBoxLeftColor,SIGNAL(stateChanged(int)),this,SLOT(activateCLeft(int)));
        connect(checkBoxRightColor,SIGNAL(stateChanged(int)),this,SLOT(activateCRight(int)));
	connect(manipModeGroup,SIGNAL(buttonClicked(QAbstractButton*)),this,SLOT(changeManipMode(QAbstractButton*)));
        connect(buttonBucketColor,SIGNAL(clicked()),this,SLOT(changeBucketColor()));

}

Interface::~Interface()
{

}

//show or hide splines;
void Interface::showSplines(int state)
{
	if (state == Qt::Unchecked)
	{

		emit showSplinesChanged(false);
		return;
	}

	if(state == Qt::Checked)
	{
		emit showSplinesChanged(true);
		return;
	}
	return;
}

//show or hide the example image
void Interface::showExampleImg(int state)
{
	if (state == Qt::Unchecked)
	{

		emit showExampleImgChanged(false);
		return;
	}

	if(state == Qt::Checked)
	{
		emit showExampleImgChanged(true);
		return;
	}
	return;
}

//activate or deactivate the blur;
void Interface::activateBlur(int state)
{
	if (state == Qt::Unchecked)
	{

		emit activateBlurChanged(false);
		return;
	}

	if(state == Qt::Checked)
	{
		emit activateBlurChanged(true);
		return;
	}
	return;
}

//activate or deactivate the left or right color
void Interface::activateCLeft(int state)
{
    if (state == Qt::Unchecked)
    {
        checkBoxRightColor->setCheckState(Qt::Checked);
        emit activateCRightChanged(true);
        emit activateCLeftChanged(false);
        return;
    }

    if(state == Qt::Checked)
    {
        emit activateCLeftChanged(true);
        return;
    }
    return;
}

void Interface::activateCRight(int state)
{
    if (state == Qt::Unchecked)
    {
        checkBoxLeftColor->setCheckState(Qt::Checked);
        emit activateCLeftChanged(true);
        emit activateCRightChanged(false);
        return;
    }

    if(state == Qt::Checked)
    {
        emit activateCRightChanged(true);
        return;
    }
    return;
}

//change the manipulation mode and mark the current mode by a sunken button
void Interface::changeManipMode(QAbstractButton* clickedButton)
{
	QList<QAbstractButton*>  buttonList = manipModeGroup->buttons();
	QList<QAbstractButton*>::iterator it;
	for(it=buttonList.begin(); it!=buttonList.end();it++)
	{
		//(*it)->setStyleSheet(styleUnpressed);
	}

	//signal that attributes have to be shown or hidden (they're shown only for SELECT_ONE::ATTRIBUTES)
	emit showAttribChanged(false);

	if(clickedButton == buttonMovePoint)
	{
		emit manipModeChanged(ManipMode_SELECT_ONE_GEOMETRY);
		//buttonMovePoint->setStyleSheet(stylePressed);

		emit showSplinesChanged(true); //we need to see the splines if we are to select them

		//QPainter painter(this);

		//QStyleOptionButton option;
		//option.initFrom(this);
		//option.state = QStyle::State_Sunken;
		////option.backgroundColor = palette().color(QPalette::Background);

		////buttonSelectOne->style()->drawPrimitive(QStyle::PE_FrameFocusRect, &option, &painter);
		//buttonSelectOne->style()->drawControl(QStyle::CE_PushButton, &option, &painter );
		//buttonDrawCatmull->setFlat(false);
		//buttonDrawLine->setFlat(false);
		//QStyle style = buttonSelectOne->style();
		//QStyleOptionButton option;
		//option.initFrom(buttonSelectOne);
		//option.state = QStyle::State_Sunken;
		////option.backgroundColor = palette().color(QPalette::Background);

		////buttonSelectOne->style()->drawPrimitive(QStyle::PE_FrameFocusRect, &option, &painter);
		//style.drawControl(QStyle::CE_PushButton, &option, &painter, buttonSelectOne);
	}
	else if(clickedButton == buttonModifAttrib)
	{
		emit manipModeChanged(ManipMode_SELECT_ONE_ATTRIB);
		//buttonModifAttrib->setStyleSheet(stylePressed);

		emit showSplinesChanged(true); //we need to see the splines if we are to select them
		emit showAttribChanged(true);
	}
	else if(clickedButton == buttonSelectMulti)
	{
		emit manipModeChanged(ManipMode_SELECT_MULTI);
		//buttonSelectMulti->setStyleSheet(stylePressed);

		if( !checkBoxShowSplines->isChecked() )
			emit showSplinesChanged(false); 
	}
	else if(clickedButton == buttonDrawCatmull)
	{
		emit manipModeChanged(ManipMode_DRAW_CATMULL);
		//buttonDrawCatmull->setStyleSheet(stylePressed);

		if( !checkBoxShowSplines->isChecked() )
			emit showSplinesChanged(false); 
	}
	else if(clickedButton == buttonPan)
	{
		emit manipModeChanged(ManipMode_PAN);
	}
	else if(clickedButton == buttonSwitchColors)
	{
		//show the attributes
		buttonModifAttrib->click();
	}
	else if(clickedButton == buttonStylize)
	{
		buttonSelectMulti->click();
	} 
	else if(clickedButton == buttonSmooth)
	{
		buttonSelectMulti->click();
	} 
	else if(clickedButton == buttonContrastIncr)
	{
		buttonSelectMulti->click();
	} 
	else if(clickedButton == buttonContrastDecr)
	{
		buttonSelectMulti->click();
	}
	else if(clickedButton == buttonSampleExample)
	{
		emit manipModeChanged(ManipMode_SELECT_ONE_GEOMETRY);
	}
	else if(clickedButton == buttonSnake)
	{
		emit manipModeChanged(ManipMode_SELECT_ONE_GEOMETRY);

		emit showSplinesChanged(true); //we need to see the splines if we are to select them
	}
        else if(clickedButton == buttonBucket)
	{
                emit manipModeChanged(ManipMode_BUCKET);

		emit showSplinesChanged(true); //we need to see the splines if we are to select them
		emit showAttribChanged(true);
        }
        else if(clickedButton == buttonBucketColor)
	{
                emit manipModeChanged(ManipMode_BUCKET);

		emit showSplinesChanged(true); //we need to see the splines if we are to select them
		emit showAttribChanged(true);
        }
        else if(clickedButton == buttonPicker)
	{
                emit manipModeChanged(ManipMode_PICKER);
                emit showSplinesChanged(true); //we need to see the splines if we are to select them
		emit showAttribChanged(true);
        }
}

void Interface::changeLeftColor()
{
    leftColor = QColorDialog::getColor(leftColor);
    emit leftColorChanged(leftColor);
    QPixmap pxmap(10,10);
    pxmap.fill(leftColor);
    QIcon ic(pxmap);
    buttonLeftColor->setIcon(ic);
}

void Interface::changeRightColor()
{
    rightColor = QColorDialog::getColor(rightColor);
    emit rightColorChanged(rightColor);
    QPixmap pxmap(10,10);
    pxmap.fill(rightColor);
    QIcon ic(pxmap);
    buttonRightColor->setIcon(ic);
}

void Interface::changeBucketColor()
{
    bucketColor = QColorDialog::getColor(bucketColor);
    emit bucketColorChanged(bucketColor);
    QPixmap pxmap(12,25);
    pxmap.fill(bucketColor);
    QIcon ic(pxmap);
    buttonBucketColor->setIcon(ic);
}

void Interface::changeBlur(int s)
{
	sigma = s;
	emit blurChanged(sigma);
}

void Interface::changeIter(int i)
{
	nbIter = i;
	emit iterChanged(nbIter);
}

void Interface::changeLifetimeThreshold(int th)
{
	lifetimeThreshold = th;
	emit lifetimeThresholdChanged(lifetimeThreshold);
}

//change a control point color
void Interface::changeControlPointLeftColor(QColor C)
{
	QColor curLeftColor = QColorDialog::getColor(C);
	if(curLeftColor.isValid())
		emit controlPointLeftColorChanged(curLeftColor);
	else
		emit controlPointLeftColorChanged(C);
}
void Interface::changeControlPointRightColor(QColor C)
{
	QColor curRightColor = QColorDialog::getColor(C);
	if(curRightColor.isValid())
		emit controlPointRightColorChanged(curRightColor);
	else
		emit controlPointRightColorChanged(C);
}

void Interface::setLeftColor(QColor C)
{
	leftColor = C;
        QPixmap pxmap(10,10);
        pxmap.fill(leftColor);
        QIcon ic(pxmap);
	buttonLeftColor->setIcon(ic);
}

void Interface::setRightColor(QColor C){
	rightColor = C;
	QPixmap pxmap(10,10);
        pxmap.fill(rightColor);
        QIcon ic(pxmap);
	buttonRightColor->setIcon(ic);
}
void Interface::setBucketColor(QColor C)
{
	bucketColor = C;
        QPixmap pxmapBucket(12,25);
        pxmapBucket.fill(bucketColor);
        QIcon icB(pxmapBucket);
        buttonBucketColor->setIcon(icB);
}

void Interface::setBlur(int s){
	sigma = s;
	sliderBlur->setValue(sigma);
}
