#ifndef INTERFACE_H
#define INTERFACE_H

#include "ui_interface.h"
#include "global_param.h"
#include <QStyle>


class Interface : public QMainWindow, public Ui::MainWindow
{
    Q_OBJECT;
    public:
        Interface(QWidget *parent = NULL);
        ~Interface();
    private:
        QColor leftColor;
        QColor rightColor;
        QColor bucketColor;

	//Style to mark that a button is pressed or not;
        QString stylePressed;
        QString styleUnpressed;
        QStyle* styleDefault;

	//I cannot create  aQButtonGroup in QtDesigner, so I'm creating it here
        QButtonGroup* manipModeGroup;
        int sigma;
        int nbIter;
        int lifetimeThreshold;
    public slots:
		//slots called by the viewer when the selection is changed
        void setLeftColor(QColor C);
        void setBucketColor(QColor C);
        void setRightColor(QColor C);
        void setBlur(int s);
		//change a control point color
        void changeControlPointLeftColor(QColor C);
        void changeControlPointRightColor(QColor C);


    signals:
            //signals sent to the viewer when the user change a value
        void leftColorChanged(QColor);
        void rightColorChanged(QColor);
        void bucketColorChanged(QColor);

            //signal to the viewer that the splines need to be shown or hidden
        void showSplinesChanged(bool);
		//signal that the example image need to be shown or hidden
		void showExampleImgChanged(bool);
            //signal to the viewer that the blur is activate or deactivate
        void activateBlurChanged(bool);
            //signal to the viewer that the left or right is activate or deactivate
        void activateCLeftChanged(bool);
        void activateCRightChanged(bool);
            //signal that attributes have to be shown or hidden (they're shown only for SELECT_ONE::ATTRIBUTES)
        void showAttribChanged(bool);
            //signal that the manipulation mode is changed
        void manipModeChanged(ManipulationMode);
            //change a control point color
        void controlPointLeftColorChanged(QColor);
        void controlPointRightColorChanged(QColor);
            //change the blur
        void blurChanged(int);
            //change the number of iterations
        void iterChanged(int);
            //change the lifetime threshold
        void lifetimeThresholdChanged(int);

    private slots:
            //slots called by the ui when the user change a value
        void changeLeftColor();
        void changeRightColor();
        void changeBucketColor();
        void changeBlur(int s);
        void changeIter(int i);
        void changeLifetimeThreshold(int th);
            //show or hide the splines; called by the Qt interface, when checkBoxShowSplines changes value
        void showSplines(int state);
		//show the example image
		void showExampleImg(int state);
            //activate or deactivate the blur.
        void activateBlur(int state);
            //activate or deactivate the left or right color
        void activateCLeft(int state);
        void activateCRight(int state);
            //change manipulation mode
        void changeManipMode(QAbstractButton* clickedButton);
};

#endif // INTERFACE_H
