/****************************************************************************

This file is part of the QGLViewer library.
Copyright (C) 2002, 2003, 2004, 2005, 2006 Gilles Debunne (Gilles.Debunne@imag.fr)
Version 2.2.1-0, released on March 8, 2006.

http://artis.imag.fr/Members/Gilles.Debunne/QGLViewer

libQGLViewer is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

libQGLViewer is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with libQGLViewer; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#include "global_param.h"
#include "poissonViewer.h"
#include "interface.h"
#include <QtGui>

int main(int argc, char** argv)
{
	// Read command lines arguments.
	QApplication application(argc,argv);

	//enable stencil
	QGLFormat format;
	format.setStencil(true);
	QGLFormat::setDefaultFormat(format);

	// Instantiate the viewer and m_interface.
	Viewer v;
	Interface m_interface;

	//connect the interface to the viewer
	QObject::connect(&m_interface,SIGNAL(leftColorChanged(QColor)),&v,SLOT(setLeftColor(QColor)));
	QObject::connect(&m_interface,SIGNAL(rightColorChanged(QColor)),&v,SLOT(setRightColor(QColor)));
        QObject::connect(&m_interface,SIGNAL(bucketColorChanged(QColor)),&v,SLOT(setBucketColor(QColor)));
	QObject::connect(&m_interface,SIGNAL(blurChanged(int)),&v,SLOT(setBlur(int)));
        QObject::connect(&m_interface,SIGNAL(iterChanged(int)),&v,SLOT(setIter(int)));
        QObject::connect(&m_interface,SIGNAL(lifetimeThresholdChanged(int)),&v,SLOT(setLifetimeThreshold(int)));
	QObject::connect(&v,SIGNAL(leftColorChanged(QColor)),&m_interface,SLOT(setLeftColor(QColor)));
	QObject::connect(&v,SIGNAL(rightColorChanged(QColor)),&m_interface,SLOT(setRightColor(QColor)));
	QObject::connect(&v,SIGNAL(blurChanged(int)),&m_interface,SLOT(setBlur(int)));
        QObject::connect(&v,SIGNAL(bucketColorChanged(QColor)),&m_interface,SLOT(setBucketColor(QColor)));
        //file menu
	QObject::connect(m_interface.actionLoad,SIGNAL(triggered()),&v,SLOT(loadXml()));
        QObject::connect(m_interface.actionSave_as_Png,SIGNAL(triggered()),&v,SLOT(savePng()));
        QObject::connect(m_interface.actionSave_as_XML,SIGNAL(triggered()),&v,SLOT(saveXml()));
        QObject::connect(m_interface.actionKeyframing,SIGNAL(triggered()),&v,SLOT(interpolate()));
        QObject::connect(m_interface.actionLoad_Bitmap,SIGNAL(triggered()),&v,SLOT(loadExampleImg()));
        //edit menu
        QObject::connect(m_interface.action_Copy,SIGNAL(triggered()),&v,SLOT(copySplines()));
        QObject::connect(m_interface.action_Paste,SIGNAL(triggered()),&v,SLOT(pasteSplines()));
        QObject::connect(m_interface.action_Cut,SIGNAL(triggered()),&v,SLOT(cutSplines()));
        QObject::connect(m_interface.action_Copy_Blur,SIGNAL(triggered()),&v,SLOT(copyBlur()));
        QObject::connect(m_interface.action_Paste_Blur,SIGNAL(triggered()),&v,SLOT(pasteBlur()));

	//demand to change a control point color
	QObject::connect(&v,SIGNAL(changeControlPointLeftColor(QColor)),&m_interface,SLOT(changeControlPointLeftColor(QColor)));
	QObject::connect(&v,SIGNAL(changeControlPointRightColor(QColor)),&m_interface,SLOT(changeControlPointRightColor(QColor)));
	QObject::connect(&m_interface,SIGNAL(controlPointLeftColorChanged(QColor)), &v,SLOT(setControlPointLeftColor(QColor)));
	QObject::connect(&m_interface,SIGNAL(controlPointRightColorChanged(QColor)), &v,SLOT(setControlPointRightColor(QColor)));

	//demand to show or hide the splines
	QObject::connect(&m_interface,SIGNAL(showSplinesChanged(bool)), &v,SLOT(showSplines(bool)));
        //demand to show or hide attributes
	QObject::connect(&m_interface,SIGNAL(showAttribChanged(bool)), &v,SLOT(showAttrib(bool)));
	//show or hide example
	QObject::connect(&m_interface,SIGNAL(showExampleImgChanged(bool)), &v,SLOT(showExampleImg(bool)));
        //demand to activate or deactivate the blur
        QObject::connect(&m_interface,SIGNAL(activateBlurChanged(bool)), &v,SLOT(activateBlur(bool)));
        //demand to activate or deactivate the left or right color
        QObject::connect(&m_interface,SIGNAL(activateCLeftChanged(bool)), &v,SLOT(activateCLeft(bool)));
        QObject::connect(&m_interface,SIGNAL(activateCRightChanged(bool)), &v,SLOT(activateCRight(bool)));
	//signal that the manipulation mode is changed
	QObject::connect(&m_interface,SIGNAL(manipModeChanged(ManipulationMode)), &v,SLOT(changeManipMode(ManipulationMode)));

        //zoom in and out
        QObject::connect(m_interface.buttonZoomIn,SIGNAL(clicked()),&v,SLOT(zoomIn()));
        QObject::connect(m_interface.buttonZoomOut,SIGNAL(clicked()),&v,SLOT(zoomOut()));
        //spline smoothing
        QObject::connect(m_interface.buttonSmooth,SIGNAL(clicked()),&v,SLOT(smoothSplines()));
        //spline stylization
        QObject::connect(m_interface.buttonStylize,SIGNAL(clicked()),&v,SLOT(stylizeSplines()));
        //contrast
        QObject::connect(m_interface.buttonContrastIncr,SIGNAL(clicked()),&v,SLOT(increaseContrast()));
        QObject::connect(m_interface.buttonContrastDecr,SIGNAL(clicked()),&v,SLOT(decreaseContrast()));
        //color switch
        QObject::connect(m_interface.buttonSwitchColors,SIGNAL(clicked()),&v,SLOT(switchColors()));
		//color sampling in examle image
        QObject::connect(m_interface.buttonSampleExample,SIGNAL(clicked()),&v,SLOT(sampleFromExample()));
		//snake
        QObject::connect(m_interface.buttonSnake,SIGNAL(clicked()),&v,SLOT(iterSnake()));

	// Make the viewer and m_interface windows visible on screen.
	v.setWindowTitle("TruePoisson");
	
	m_interface.show();

	/*Viewer v1;
	v1.setWindowTitle("v1");
	Viewer v2;
	v2.setWindowTitle("v2");
	v1.show();
	v2.show();*/

	v.show();

#if QT_VERSION < 0x040000
	// Set the viewer as the application main widget.
	application.setMainWidget(&v);
#endif

	// Run main loop.
	return application.exec();
}
