#include "poissonViewer.h"
#include <qimage.h>
#include <QKeyEvent>
#include <QFileDialog>

# include <QMenu>
# include <QMouseEvent>
//#include <qcursor.h>
//#include <qmap.h>

using namespace std;

void Viewer::selectMulti_keyPressEvent(QKeyEvent* e)
{
    	switch (e->key())
	{
		//selection
	case Qt::Key_Delete:
		splines.deleteSelection();
		poissonSolver->slowUpdate();
		break;
	default: 
		QGLViewer::keyPressEvent(e);
	}
}

void Viewer::selectMulti_mousePressEvent(QMouseEvent* e)
{
	switch ( e->button())
	{
	case Qt::LeftButton:
		selectMulti_leftButton(e);
		break;
	case Qt::RightButton:
		selectMulti_rightButton(e);
		break;
	default:
		QGLViewer::mousePressEvent(e);
	}
}

void Viewer::selectMulti_mouseMoveEvent(QMouseEvent* e)
{
	double x,y;

	///////////////////////////////////////////////////////////////////////////////////////
	//if CTRL + LEFT_BUTTON, select underlying splines
	//can't check for left button, the returned value is always Qt::NoButton for mouse move events.
	if( e->modifiers() == Qt::ControlModifier) //CTRL + LEFT_BUTTON
	{
		x = e->x();
		y = height()-e->y();
		GLuint buff[64] = {0};
		GLint hits;
		double mouseWindow = 20; //define a big area around the mouse cursor
		//get all the splines touched by the mouse
		hits = getPickingHits(MANIP_MODE, ClickType_CTRL_LEFT, x, y, mouseWindow, buff);
		for (int i = 0; i < hits; i++)
		{
			splines.addToSelection(buff[i * 4 + 3]);
		}
		//save mouse path
		Point mousePoint;
		x = e->x()/(0.5*width())-1;
		y = 1-e->y()/(0.5*height());
		x/=zoom;
		y/=zoom;
		x-= center_x;
		y-= center_y;
		mousePoint.setX(x); mousePoint.setY(y);
		selectPath.push_back(mousePoint);
		return;
		//no solver update
	}

	///////////////////////////////////////////////////////////////////////////////////////
	// if just LEFT_BUTTON, we're dragging the selected splines
	if(splines.isDraggingSpline())
	{
		x = e->x()/(0.5*width())-1;
		y = 1-e->y()/(0.5*height());
		x/=zoom;
		y/=zoom;
		x-= center_x;
		y-= center_y;

		splines.dragSpline(x,y);
		poissonSolver->fastUpdate();
		return;
	}
	//else, default
	QGLViewer::mouseMoveEvent(e);
}

void Viewer::selectMulti_mouseReleaseEvent(QMouseEvent* e)
{
	//deactivate the dragging mode
	if(splines.isDraggingSpline())
	{
		splines.setDragPoint(0,0,false);
	}
	//clear the mouse path, if we where using CTRL+LEFT_BUTTON selection mode
	selectPath.clear();
	QGLViewer::mouseReleaseEvent(e);
}


//*****************************************************
void Viewer::selectMulti_leftButton(QMouseEvent* e)
{
	double x,y;
	x = e->x();
	y = height()-e->y();

	GLuint buff[64] = {0};
	GLint hits;
	double mouseWindow = 5;
	///////////////////////////////////////////////////////////////////////////////////////
	if ( e->modifiers() == Qt::ShiftModifier) //SHIFT + LEFT_BUTTON
	{
		hits = getPickingHits(MANIP_MODE, ClickType_SHIFT_LEFT, x, y, mouseWindow, buff);
		for (int i = 0; i < hits; i++)
		{
			//a second click deselects the spline
			splines.addRemoveToSelection(buff[i * 4 + 3]);
		}
	}
	///////////////////////////////////////////////////////////////////////////////////////
	else if ( e->modifiers() == Qt::NoModifier) //just LEFT_BUTTON
	{
		hits = getPickingHits(MANIP_MODE, ClickType_LEFT_BUTTON, x, y, mouseWindow, buff);
		//if nothing was touched, deselect
		if(hits == 0)
		{
			splines.deselectAll();
		}

		switch(buff[(hits-1)*4+3])
		{
		case 0:
			splines.deselectAll();
			break;
		case 1: ;//it's in the bounding box
		case 2: //it's on a selected spline
			double x_d,y_d;
			x_d = x/(0.5*width())-1;
			y_d = (y/height()-1)*2+1;
			x_d/=zoom;
			y_d/=zoom;
			x_d-= center_x;
			y_d-= center_y;
			splines.setDragPoint(x_d, y_d, true);
			break;
		default:
			;
		}
	}
}

void Viewer::selectMulti_rightButton(QMouseEvent* e)
{
	if(!splines.areMultiSplinesSelected())
		return;

	QMenu menu( this );
	//menu.addSeparator();
	QMap<QAction*, int> menuMap;
	menuMap[menu.addAction("Increase/decrease blur")] = 1;
	//menuMap[menu.addAction("Delete control point")] = 2;
	
	QDialog* blurDialog = new QDialog( this );
	blurDialog->resize(100,20);
	QSlider* slider= new QSlider(Qt::Horizontal, blurDialog);
	slider->setMinimum(-MAX_BLUR);
	slider->setMaximum(MAX_BLUR);
int old_val, shift_val;
	QAction* action = menu.exec(e->globalPos());
	if(action)
	{
		switch ( menuMap[action] )
		{
			//blur
		case 1:
			slider->setValue(0);
			old_val = slider->value();
			blurDialog->move(e->globalPos());
			blurDialog->exec();
			shift_val = slider->value()-old_val;
			setGlobalBlur(shift_val); 
			break;
		}

		poissonSolver->slowUpdate();
	}
	return;
}

