#include "poissonViewer.h"
#include <qimage.h>
#include <QKeyEvent>
#include <QFileDialog>

# include <QMenu>
# include <QMouseEvent>
//#include <qcursor.h>
//#include <qmap.h>

using namespace std;


//manipulation functions
void Viewer::selectOne_keyPressEvent(QKeyEvent* e)
{
	int id ;
	switch (e->key())
	{
		//selection
	case Qt::Key_Delete:
		id = splines.getSelected();
		if (id>=0)
		{
			splines.removeSpline(id);
			splines.setSelected(-1);
			poissonSolver->slowUpdate();
		}
		break;
	default: 
		QGLViewer::keyPressEvent(e);
	}
}

void Viewer::selectOne_mousePressEvent(QMouseEvent* e)
{
	switch ( e->button())
	{
	case Qt::LeftButton:
		selectOne_leftButton(e);
		break;
	case Qt::RightButton:
		selectOne_rightButton(e);
		break;
	default:
		QGLViewer::mousePressEvent(e);
	}
}

void Viewer::selectOne_mouseMoveEvent(QMouseEvent* e)
{
	double x,y;
	x = e->x()/(0.5*width())-1;
	y = 1-e->y()/(0.5*height());
	x/=zoom;
	y/=zoom;
	x-= center_x;
	y-= center_y;
	if(splines.isDraggingSpline())
	{
		splines.dragSpline(x,y);
		poissonSolver->fastUpdate();
	}
	else if(splines.movePoint(x,y)){
		poissonSolver->fastUpdate();
	}

	QGLViewer::mouseMoveEvent(e);
}

void Viewer::selectOne_mouseReleaseEvent(QMouseEvent* e)
{
	if(splines.isDraggingSpline())
	{
		splines.setDragPoint(0,0,false);
                poissonSolver->slowUpdate();
	}
	else if(splines.getMovingPoint()>=0)
	{
		splines.releaseMovingPoint();
                poissonSolver->slowUpdate();
	}
}


void Viewer::selectOne_leftButton(QMouseEvent* e)
{
	double x,y;
	x = e->x();
	y = height()-e->y();

	double mouseWindow = 5;

	GLuint buff[64] = {0};
	GLint hits = getPickingHits(MANIP_MODE, ClickType_LEFT_BUTTON, x, y, mouseWindow, buff);

	//if nothing was touched, deselect
	if(hits == 0)
	{
		splines.setSelected(-1);
		return;
	}

	//something was touched

	//if no spline is selected, we select the spline
	if(splines.getSelected()==-1)
	{
		splines.setSelected(buff[3]);
		return;
	}
	//if a spline is selected, test the uppermost hit
	//if it's an outside point, deselect
	if (buff[(hits-1)*4+3] == 0)
	{
		splines.setSelected(-1);
		return;
	}
	/*if (buff[(hits-1)*4+3] == 1)
	{
	//it's inside the bounding box, but not on the spline
	double x_d,y_d;
	x_d = x/(0.5*width())-1;
	y_d = (y/height()-1)*2+1;
	x_d/=zoom;
	y_d/=zoom;
	x_d-= center_x;
	y_d-= center_y;
	splines.setDragPoint(x_d, y_d, true);
	return;
	}*/
	//starting now, the behaviour changes for each SPLINE_MANIP
	if(MANIP_MODE == ManipMode_SELECT_ONE_GEOMETRY)
	{
		switch (buff[(hits-1)*4+3])
		{
		case 1:
		case 2: //it's on the spline, but not near control points
			double x_d,y_d;
			x_d = x/(0.5*width())-1;
			y_d = (y/height()-1)*2+1;
			x_d/=zoom;
			y_d/=zoom;
			x_d-= center_x;
			y_d-= center_y;
			splines.setDragPoint(x_d, y_d, true);
			break;
		default: //it's on a control point
                        const unsigned int tagLength = 10000;
			splines.setMovingPoint(buff[(hits-1)*4+3]-tagLength);
		}
		return;
	}

	if(MANIP_MODE == ManipMode_SELECT_ONE_REDRAW)
	{
		switch (buff[(hits-1)*4+3])
		{
		case 2: //it's on the spline, but not near control points
			break;
		default: //it's on a control point
			;
		}
		return;
	}

	if(MANIP_MODE == ManipMode_SELECT_ONE_ATTRIB)
	{
		const unsigned int tagLength = 10000;
		QDialog* blurDialog = new QDialog( this );
		blurDialog->resize(100,20);
		QSlider* slider= new QSlider(Qt::Horizontal, blurDialog);
		slider->setMinimum(0);
		slider->setMaximum(MAX_BLUR);
		switch (buff[(hits-1)*4+3]/tagLength)
		{
		case 0: //it's on the spline, but not near control points
			break;
		case 1://blur;
			ATTRIB_ID = (buff[(hits-1)*4+3]%tagLength);
			slider->setValue(splines.getSigma(ATTRIB_ID));
			blurDialog->move(e->globalPos());
			blurDialog->exec();
			splines.setSigma(slider->value(), ATTRIB_ID);
			break;
		case 2://left color
			ATTRIB_ID = (buff[(hits-1)*4+3]%tagLength);
			emit changeControlPointLeftColor(splines.getControlPointLeftColor(ATTRIB_ID));
			break;
		case 3: //right color
			ATTRIB_ID = (buff[(hits-1)*4+3]%tagLength);
			emit changeControlPointRightColor(splines.getControlPointRightColor(ATTRIB_ID));
			break;
		default:
			;
		}
		poissonSolver->slowUpdate();
		return;
	}

        if(MANIP_MODE == ManipMode_BUCKET)
	{
                const unsigned int tagLength = 10000;
		switch (buff[(hits-1)*4+3]/tagLength)
		{
		case 0: //it's on the spline, but not near control points
			break;
		case 1://blur;
			break;
		case 2://left color
			ATTRIB_ID = (buff[(hits-1)*4+3]%tagLength);
                        setControlPointLeftColor(bucketColor);
			break;
		case 3: //right color
			ATTRIB_ID = (buff[(hits-1)*4+3]%tagLength);
                        setControlPointRightColor(bucketColor);
			break;
		default:
			;
		}
		poissonSolver->slowUpdate();
		return;
        }
}


void Viewer::selectOne_rightButton(QMouseEvent* e)
{
	double x,y;
	x = e->x();
	y = height()-e->y();

	double mouseWindow = 5;
	GLuint buff[64] = {0};
	GLint hits = getPickingHits(MANIP_MODE, ClickType_RIGHT_BUTTON, x, y, mouseWindow, buff);

	//if nothing was touched or if no spline is selected, nothing is done
	if( (hits == 0) || (splines.getSelected()==-1) )
	{
		return;
	}
	//if a spline is selected, test the uppermost hit
	//if it's an outside point, or it's inside the bounding box, but not on the spline
	//nothing is done
	if (buff[(hits-1)*4+3] <= 1)
	{
		return;
	}


	//starting now, the behaviour changes for each SPLINE_MANIP
	if(MANIP_MODE == ManipMode_SELECT_ONE_GEOMETRY)
	{
		QMenu menu( this );
		//menu.addSeparator();
		QMap<QAction*, int> menuMap;
		menuMap[menu.addAction("Add control point")] = 1;
		menuMap[menu.addAction("Delete control point")] = 2;
                menuMap[menu.addAction("Split the curve")] = 3;
		QAction* action = menu.exec(e->globalPos());
		if(action)
		{
                        const unsigned int tagLength = 10000;
                        double x_d,y_d,t;
                        x_d = x/(0.5*width())-1;
                        y_d = (y/height()-1)*2+1;
                        x_d/=zoom;
                        y_d/=zoom;
                        x_d-= center_x;
                        y_d-= center_y;
                        int id = buff[(hits-1)*4+3];
                        if(id<(int)tagLength){//if we are on the curve
                            t = (double)(id % Catmull::NB_STEPS)/(double)Catmull::NB_STEPS;
                            id = 3*(id/Catmull::NB_STEPS-1)+2;
                        }
                        else{//we are on a control point
                            id = 3*((id-tagLength)/3)+2;
                            t = 0.1;
                        }
			switch ( menuMap[action] )
			{
                        case 1:
			//we only insert a point with bacward and forward tangents, at the middle of a previous Bezier, between P1 and P2
				splines.insertPoint(x_d, y_d, id, t);
                                break;
			case 2:
				splines.erasePoint(id-2);
                                break;
			case 3:
                                splines.split( x_d, y_d, id, t);
                                break;
                        default:
                                break;
                        }
			poissonSolver->slowUpdate();
		}
		return;
	}

	if(MANIP_MODE == ManipMode_SELECT_ONE_REDRAW)
	{
		switch (buff[(hits-1)*4+3])
		{
		case 2: //it's on the spline, but not near control points
			break;
		default: //it's on a control point
			break;
		}
		return;
	}

	if(MANIP_MODE == ManipMode_SELECT_ONE_ATTRIB)
	{
		QMenu menu( this );
		QMap<QAction*, int> menuMap;
		menuMap[menu.addAction("Add Left Color")] = 1;
		menuMap[menu.addAction("Delete Left Color")] = 2;
		menu.addSeparator();
		menuMap[menu.addAction("Add Right Color")] = 3;
		menuMap[menu.addAction("Delete Right Color")] = 4;
		menu.addSeparator();
		menuMap[menu.addAction("Add Blur")] = 5;
		menuMap[menu.addAction("Delete Blur")] = 6;
		QAction* action = menu.exec(e->globalPos());

		if(action)
		{
			bool attribIsSet = false;
			AttribType attribType;
			int id = buff[(hits-1)*4+3]-Catmull::NB_STEPS;

			switch ( menuMap[action])
			{
				//add part:
			case 1:
				attribType = COLOR_LEFT;
				attribIsSet = true;
                               
			case 3:
				if(!attribIsSet)
				{
					attribType = COLOR_RIGHT;
					attribIsSet = true;
				}
                              
			case 5:
				if(!attribIsSet)
				{
					attribType = BLUR;
					attribIsSet = true;
				}
				splines.insertAttribPoint(id, attribType);
				//std::cerr<<"Add "<<attribType<<" at point "<<id<<"\n";
				break;
				//delete part:
			case 2:
				attribType = COLOR_LEFT;
				attribIsSet = true;
                              
			case 4:
				if(!attribIsSet)
				{
					attribType = COLOR_RIGHT;
					attribIsSet = true;
				}
                                
			case 6:
				if(!attribIsSet)
				{
					attribType = BLUR;
					attribIsSet = true;
				}
				splines.eraseAttribPoint(id, attribType);
				//std::cerr<<"Add "<<attribType<<" at point "<<id<<"\n";
				break;
			}
			poissonSolver->slowUpdate();
		}//end if(action)
	} //end if(MANIP_MODE == ManipMode_SELECT_ONE_ATTRIB)

        if(MANIP_MODE == ManipMode_BUCKET)
	{
		QMenu menu( this );
		QMap<QAction*, int> menuMap;
		menuMap[menu.addAction("Add Left Color")] = 1;
		menuMap[menu.addAction("Delete Left Color")] = 2;
		menu.addSeparator();
		menuMap[menu.addAction("Add Right Color")] = 3;
		menuMap[menu.addAction("Delete Right Color")] = 4;
		QAction* action = menu.exec(e->globalPos());

		if(action)
		{
			bool attribIsSet = false;
			AttribType attribType;
			int id = buff[(hits-1)*4+3]-Catmull::NB_STEPS;

			switch ( menuMap[action])
			{
				//add part:
			case 1:
				attribType = COLOR_LEFT;
				attribIsSet = true;
                               
			case 3:
				if(!attribIsSet)
				{
					attribType = COLOR_RIGHT;
					attribIsSet = true;
				}
                                splines.insertAttribPoint(id, attribType);
				break;
				//delete part:
			case 2:
				attribType = COLOR_LEFT;
				attribIsSet = true;
                              
			case 4:
				if(!attribIsSet)
				{
					attribType = COLOR_RIGHT;
					attribIsSet = true;
				}
                                splines.eraseAttribPoint(id, attribType);
				break;
			}
			poissonSolver->slowUpdate();
		}//end if(action)
	} //end if(MANIP_MODE == ManipMode_SELECT_ONE_ATTRIB)
}

//auxiliary functions
GLint Viewer::getPickingHits(ManipulationMode splineManip, ClickType clickType, double x, double y, double mouseWindow, GLuint* buff)
{
	//GLuint buff[64] = {0};
	GLint hits, view[4];
	/*
	This choose the buffer where store the values for the selection data
	*/
	glSelectBuffer(64, buff);
	/*
	This retrieve info about the viewport
	*/
	glGetIntegerv(GL_VIEWPORT, view);

	/*
	Switching in selecton mode
	*/
	glRenderMode(GL_SELECT);

	/*
	Clearing the name's stack
	This stack contains all the info about the objects
	*/
	glInitNames();

	/*
	Now fill the stack with one element (or glLoadName will generate an error)
	*/
	glPushName(0);

	/*
	Now modify the vieving volume, restricting selection area around the cursor
	*/
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	/*
	restrict the draw to an area around the cursor
	*/
	gluPickMatrix(x, y, mouseWindow, mouseWindow, view);
	//gluPerspective(60, 1.0, 0.0001, 1000.0);

	/*
	Draw the objects onto the screen
	*/
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glScalef(zoom,zoom,0);
	glTranslatef(center_x,center_y,0.0);

	/*
	draw only the names in the stack, and fill the array
	*/
	splines.drawWithNameTags(splineManip, clickType);

	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	/*
	get number of objects drawed in that area
	and return to render mode
	*/
	hits = glRenderMode(GL_RENDER);
	glMatrixMode(GL_MODELVIEW);

	//printf("%d hits:\n", hits);
	//for (int i = 0; i < hits; i++)
	//	printf(	"Number: %d\n"
	//	"Min Z: %d\n"
	//	"Max Z: %d\n"
	//	"Name on stack: %d\n",
	//	(GLubyte)buff[i * 4],
	//	(GLubyte)buff[i * 4 + 1],
	//	(GLubyte)buff[i * 4 + 2],
	//	(GLubyte)buff[i * 4 + 3]
	//);

	//printf("\n");

	return hits;
}
