#include "matrix.h"


 Matrix::Matrix(int N,int M, double defVal)
{ 
    N_row = N;
    N_col = M;
    createComponent(N,M);
    for(int i=0;i<N;i++){
        for(int j=0;j<M;j++){
	        component[i][j] = defVal;
        }
    }
}

 Matrix::Matrix(int N,int M)
{ 
    N_row = N;
    N_col = M;
    createComponent(N,M);
}

 Matrix::~Matrix()
{
    freeComponent(); // Maxime : Im not sure this is necessary (lack of skill in C++)
}


int Matrix::getNbRow(){return N_row;}
int Matrix::getNbCol(){return N_col;}


 Matrix Matrix::transpose() const
{    
    Matrix res = Matrix(N_col,N_row);

    for(int i = 0;i < N_row;++i){
        for(int j = 0;j < N_col;++j){
	        res(j,i) = component[i][j];
        }
    }

    return res;
}

 Matrix Matrix::inverse() const
{
		
		Matrix result = Matrix(N_row,N_row,0);
		
		if (N_row != N_col)  {
		    std::cout << "ERROR in Matrix::inverse : Dimension conflict, Matrix(N_row,N_row,0.0) returned" << std::endl;		    
		}
		else {
		    int N = N_row;
    
		    Matrix proxy = Matrix(N,2*N,0);
		    Matrix vec = Matrix(1,2*N,0);

		    for(int i=0;i<N;i++){
			    for(int j=0;j<N;j++){
			    	proxy(i,j)   = component[i][j];
			    	proxy(i,j+N) = (i==j) ? 1 : 0;
			    }
		    }
   
		    for(int row=0;row<N;row++){
			    if (proxy(row,row) == 0){
			    	bool cont = true;
			    	for(int r=row+1;(r<N)&&cont;r++){  //0 remplacer par row 
						cont = (proxy(r,row) == 0);
			    		if(!cont){
			    			proxy.swap_rows(r,row);
			    		}
			    	}
			    }
								
			    proxy.multiply_row(row,1.0/proxy(row,row));
			    
				vec = proxy.get_vector_from_row(row);
	
			    for(int r=0;r<N;r++){
				    if (r==row){
				    	r++;
				    }
				    if (r<N) proxy.add_vector_to_row(r,-vec*proxy(r,row));
			    }
		    }  

		    for(int i=0;i<N;i++){
			    for(int j=0;j<N;j++){
			    	result(i,j) = proxy(i,j+N);
			    }
		    }
		}
		
		return result;

}

Matrix& Matrix::operator=(const Matrix& m)
{
    if (this!=&m){
        for(int i=0;i<N_row;i++){
	        for(int j=0;j<N_col;j++){
	            component[i][j] = m(i,j);
	        }
        }
    }
    return *this;
}

Matrix& Matrix::operator+=(const Matrix& m)
{
    for(int i=0;i<N_row;i++){
        for(int j=0;j<N_col;j++){
	        component[i][j] += m(i,j);
        }
    }
    return *this;
}

 Matrix& Matrix::operator-=(const Matrix& m)
{
    for(int i=0;i<N_row;i++){
        for(int j=0;j<N_col;j++){
	        component[i][j] -= m(i,j);
        }
    }
    return *this;
}

Matrix& Matrix::operator*=(const double& lambda)
{
    for(int i=0;i<N_row;i++){
        for(int j=0;j<N_col;j++){
	        component[i][j] *= lambda;
        }
    }
    return *this;
}

 Matrix& Matrix::operator/=(const double& lambda)
{
    if (lambda == 0) {
        std::cout << "ERROR in Matrix::operator/= : lambda param equal to 0" << std::endl;
    }
    else {
        for(int i=0;i<N_row;i++){
            for(int j=0;j<N_col;j++){
	            component[i][j] /= lambda;
            }
        }
    }
    return *this;
}
 Matrix Matrix::operator-() const
{
    Matrix res = Matrix(N_row,N_col);
    for(int i=0;i<N_row;i++){
        for(int j=0;j<N_col;j++){
	        res(i,j) = -component[i][j];
        }
    }
    return res;
}

//@}

//@{
//! Classical access operator.

 const double& Matrix::operator()(const int i,const int j) const
{
    return component[i][j];
}
 double& Matrix::operator()(const int i,const int j)
{  
    return component[i][j];
}

//@}


//@{
//! Row and column operators

 void Matrix::swap_rows(const int row1,const int row2)
{
    for(int j=0;j<N_col;j++){
        std::swap(component[row1][j],component[row2][j]);
    }  
}

 void Matrix::multiply_row(const int row,const double lambda)
{
    for(int j=0;j<N_col;j++){
        component[row][j] *= lambda;
    }   
}

 Matrix Matrix::get_vector_from_row(const int row) const
{
    Matrix res = Matrix(1,N_col);
    
    for(int j=0;j<N_col;j++){
        res(0,j) = component[row][j];
    }

    return res;
} 

 void Matrix::add_vector_to_row(const int row,const Matrix& m)
{
    if ((m.N_row != 1) || (m.N_col != N_col)) {
        std::cout << "ERROR in Matrix::add_vector_to_row : Dimension conflict" << std::endl;
    }
    else {
        
        for(int j=0;j<N_col;j++){
            component[row][j] += m(0,j);
        }
    }
}

//@}

void Matrix::display()
{
    for (int i = 0; i<N_row; i++) {
        for (int j = 0; j<N_col; j++) {
            std::cout << component[i][j] << "\t";
        }
        std::cout << std::endl;
    }
}
    
 void Matrix::createComponent(int N,int M)
{
    component = new double* [N];
    for(int i=0 ; i<N ; i++){
        component[i] = new double[M];
    }

}

 void Matrix::freeComponent() 
{
    for(int i=0 ; i<N_row ; i++){
        delete[] component[i];
    }
    delete component;
}




  
//@{
//! Classical operator.

Matrix operator+(const Matrix& m1,const Matrix& m2)
{
    Matrix res = Matrix(m1.N_row,m1.N_col);
	
	res = m1;

    res += m2;

    return res;
}


Matrix operator-(const Matrix& m1,const Matrix& m2)
{
    Matrix res = m1; //Matrix(m1.N_row,m1.N_col); pourquoi ca marche pas??

	res = m1;

    res -= m2;

    return res;
}

Matrix operator*(const double lambda,const Matrix& m)
{
    Matrix res = Matrix(m.N_row,m.N_col);
	
	res = m;

    res *= lambda;

    return res;
}

Matrix operator*(const Matrix& m, const double lambda)
{
    Matrix res = Matrix(m.N_row,m.N_col);
	
	res = m;

    res *= lambda;

    return res;
}

// Exception must be raised in case of dimension conflict
Matrix operator*(const Matrix& m1,const Matrix& m2)
{
    int i,j,k;
    int N = m1.N_row;
    int P = m1.N_col;
    int Q = m2.N_col;
    
	
	Matrix res = Matrix(m1.N_row,m2.N_col,0);
    if (m1.N_col != m2.N_row)  {
        std::cout << "ERROR in Matrix::operator* : Dimension conflict, Matrix(m1.N_row,m2.N_col,0.0) returned" << std::endl;
	}else{
		double scale;
  
        for(j=0;j<Q;j++){
            for(k=0;k<P;k++){
      
	            scale = m2(k,j);

	            for(i=0;i<N;i++){
	                res(i,j) += (m1(i,k) * scale);
	            }
            }
        }
     }
	 return res;
}

Matrix operator/(const Matrix& m,const double lambda)
{
    Matrix res = Matrix(m.N_row,m.N_col);
	
	res = m;

    res /= lambda;

    return res;
}



