#include "solver.h"
#include <QImage>
#include <iostream>
using namespace std;

Solver::Solver(int width, int height, int nI, double z, double cx, double cy)
{
    _w = width;
    _h = height;

    _gxBuffers.resize(1);
    _gxBufferFBOs.resize(1);
    _gyBuffers.resize(1);
    _gyBufferFBOs.resize(1);
    _divBuffers.resize(1);
    _divBufferFBOs.resize(1);
    _resultBuffers.resize(1);
    _resultBufferFBOs.resize(1);
    _tempJacobiBuffers.resize(1);
    _tempJacobiBufferFBOs.resize(1);
    _blurBuffers.resize(1);
    _blurBufferFBOs.resize(1);
    _colorBuffers.resize(1);
    _colorBufferFBOs.resize(1);
    _blurAmountBuffers.resize(1);
    _blurAmountBufferFBOs.resize(1);
    deepness = 1;

    nbIter = nI;
    activateBlur = false;
    blurStep = 2;

    zoom = z;
    center_x = cx;
    center_y = cy;

    deepness_debug=0;
}

Solver::~Solver()
{
    for(int d=0;d<deepness;d++)
    {
        glDeleteTextures(1,&_gxBuffers[d]);
        glDeleteFramebuffersEXT(1,&_gxBufferFBOs[d]);
        glDeleteTextures(1,&_gyBuffers[d]);
        glDeleteFramebuffersEXT(1,&_gyBufferFBOs[d]);
        glDeleteTextures(1,&_divBuffers[d]);
        glDeleteFramebuffersEXT(1,&_divBufferFBOs[d]);
        glDeleteTextures(1,&_resultBuffers[d]);
        glDeleteFramebuffersEXT(1,&_resultBufferFBOs[d]);
        glDeleteTextures(1,&_tempJacobiBuffers[d]);
        glDeleteFramebuffersEXT(1,&_tempJacobiBufferFBOs[d]);
        glDeleteTextures(1,&_blurBuffers[d]);
        glDeleteFramebuffersEXT(1,&_blurBufferFBOs[d]);
        glDeleteTextures(1,&_colorBuffers[d]);
        glDeleteFramebuffersEXT(1,&_colorBufferFBOs[d]);
        glDeleteTextures(1,&_blurAmountBuffers[d]);
        glDeleteFramebuffersEXT(1,&_blurAmountBufferFBOs[d]);
    }

    glDeleteTextures(1,&_colorStencilBuffer);
    glDeleteFramebuffersEXT(1,&_referenceBufferFBO);
    glDeleteTextures(1,&_referenceBuffer);
    glDeleteFramebuffersEXT(1,&_tempBufferFBO);
    glDeleteTextures(1,&_tempBuffer);
    
    if(w!=NULL) delete [] w;
    if(h!=NULL) delete [] h;
}

void Solver::setSplines(SplineSet* s)
{
    splines = s;
}

void Solver::init()
{
    // offscreen buffers
    deepness = (int)(log((double)_w)/log(2.0)+0.5)-3;
    _gxBuffers.resize(deepness);
    _gxBufferFBOs.resize(deepness);
    _gyBuffers.resize(deepness);
    _gyBufferFBOs.resize(deepness);
    _divBuffers.resize(deepness);
    _divBufferFBOs.resize(deepness);
    _resultBuffers.resize(deepness);
    _resultBufferFBOs.resize(deepness);
    _tempJacobiBuffers.resize(deepness);
    _tempJacobiBufferFBOs.resize(deepness);
    _blurBuffers.resize(deepness);
    _blurBufferFBOs.resize(deepness);
    _colorBuffers.resize(deepness);
    _colorBufferFBOs.resize(deepness);
    _blurAmountBuffers.resize(deepness);
    _blurAmountBufferFBOs.resize(deepness);
    w = new int[deepness+1];
    h = new int[deepness+1];
    GLint depth = GL_RGBA16F_ARB;
    float borderCol[4]={0.5,0.5,0.5,1};

    w[0] = _w;
    h[0] = _h;
    for(int d=0;d<deepness;d++)
    {
        //x derivative
        glGenTextures(1,&_gxBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _gxBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );	
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_BORDER);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_BORDER);	
        glTexParameterfv(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_BORDER_COLOR,borderCol);
        glGenFramebuffersEXT(1,&_gxBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_gxBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_gxBuffers[d],0);
    
        //y derivative
        glGenTextures(1,&_gyBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _gyBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );	
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_BORDER);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_BORDER);
        glTexParameterfv(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_BORDER_COLOR,borderCol);
        glGenFramebuffersEXT(1,&_gyBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_gyBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_gyBuffers[d],0);
    
        //divergence
        glGenTextures(1,&_divBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _divBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_BORDER);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_BORDER);
        glTexParameterfv(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_BORDER_COLOR,borderCol);
        glGenFramebuffersEXT(1,&_divBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_divBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_divBuffers[d],0);
    
        //result
        glGenTextures(1,&_resultBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _resultBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );		
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
        glGenFramebuffersEXT(1,&_resultBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[d],0);
    
        //temporary buffer for Jacobi iterations
        glGenTextures(1,&_tempJacobiBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _tempJacobiBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );		
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
        glGenFramebuffersEXT(1,&_tempJacobiBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempJacobiBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_tempJacobiBuffers[d],0);

        //blur buffer
        glGenTextures(1,&_blurBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _blurBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );		
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
        glGenFramebuffersEXT(1,&_blurBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[d],0);

        //color constraints buffer
        glGenTextures(1,&_colorBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _colorBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA8, w[d], h[d], 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );		
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
        glGenFramebuffersEXT(1,&_colorBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_colorBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_colorBuffers[d],0);

        //constraints on the amount of blur per pixel along edges
        glGenTextures(1,&_blurAmountBuffers[d]);
        glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _blurAmountBuffers[d]);
        glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, w[d], h[d], 0, GL_RGBA, GL_FLOAT, NULL);
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );		
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
        glGenFramebuffersEXT(1,&_blurAmountBufferFBOs[d]);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurAmountBufferFBOs[d]);
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_blurAmountBuffers[d],0);

        w[d+1] = w[d]/2;
        h[d+1] = h[d]/2;
    }

    //stencil buffer for the first level of the color constraints
    glGenTextures(1, &_colorStencilBuffer);
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB, _colorStencilBuffer);
    glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_DEPTH24_STENCIL8_EXT, w[0], h[0], 0, GL_DEPTH_STENCIL_EXT, GL_UNSIGNED_INT_24_8_EXT, NULL);
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_colorBufferFBOs[0]);
    glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_TEXTURE_RECTANGLE_ARB , _colorStencilBuffer, 0);
    glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT, GL_TEXTURE_RECTANGLE_ARB , _colorStencilBuffer, 0);

    //reference buffer
    glGenTextures(1,&_referenceBuffer);
    glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _referenceBuffer);
    glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, _w, _h, 0, GL_RGBA, GL_FLOAT, NULL);
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );		
    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
    glGenFramebuffersEXT(1,&_referenceBufferFBO);
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_referenceBufferFBO);
    glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_referenceBuffer,0);
    glClearColor(0.5f,0.5f,0.5f,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);

    //temp buffer
    glGenTextures(1,&_tempBuffer);
    glBindTexture( GL_TEXTURE_RECTANGLE_ARB, _tempBuffer);
    glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, depth, _w, _h, 0, GL_RGBA, GL_FLOAT, NULL);
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameterf(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );		
    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
    glGenFramebuffersEXT(1,&_tempBufferFBO);
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
    glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_RECTANGLE_ARB,_tempBuffer,0);

    // GLSL program
    if (!divProgram.create("shaders/img_process.vert","shaders/div.frag"))
    {
        cerr<<"Div Shader : "<<divProgram.lastError()<<endl;
    }
    if (!jacobiPoissonProgram.create("shaders/img_process.vert","shaders/jacobi_poisson.frag"))
    {
        cerr<<"Jacobi Poisson Shader : "<<jacobiPoissonProgram.lastError()<<endl;
    }
    if (!jacobiLaplaceProgram.create("shaders/img_process.vert","shaders/jacobi_laplace.frag"))
    {
        cerr<<"Jacobi Laplace Shader : "<<jacobiLaplaceProgram.lastError()<<endl;
    }
    if (!restrictProgram.create("shaders/img_process.vert","shaders/restrict.frag"))
    {
        cerr<<"Restriction Shader : "<<restrictProgram.lastError()<<endl;
    }
    if (!anisotropicDiffusionProgram.create("shaders/img_process.vert","shaders/anisotropic_diffusion.frag"))
    {
        cerr<<"Diffusion Shader : "<<anisotropicDiffusionProgram.lastError()<<endl;
    }
    if (!restrictColorProgram.create("shaders/img_process.vert","shaders/restrict_color.frag"))
    {
        cerr<<"Restrict Color Shader : "<<restrictColorProgram.lastError()<<endl;
    }
    if (!blurThresholdProgram.create("shaders/img_process.vert","shaders/blur_threshold.frag"))
    {
        cerr<<"Blur Threshold Shader : "<<blurThresholdProgram.lastError()<<endl;
    }
}

void Solver::updateMultigrid()
{
    if(zoom>1){
        //--- we need color constraints on the image border ---//
        //store the current zoom and pan
        double z = zoom;
        double cx = center_x;
        double cy = center_y;
        bool blurA = activateBlur;
        //put to the initial state
        zoom = 1;
        center_x = center_y = 0;
        activateBlur = false;
        //compute the solution at initial scale to obtain border colors
        initGradients();
        initColorConstraints(false);
		iterJacobiColor();
        //put back to the current state
        zoom = z;
        center_x = cx;
        center_y = cy;
        activateBlur = blurA;
        //compute the gradient and color constraints at current state, using borders
        initGradients();
        initBlurConstraints();
        initColorConstraints(true);
    }
    else{
        initGradients();
        initBlurConstraints();
        initColorConstraints(false);
    }
}

//update the reference buffer with the current solution
void Solver::updateReference()
{
    double z = zoom;
    double cx = center_x;
    double cy = center_y;
    int nbI = nbIter;
    //put to the initial state
    zoom = 1;
    center_x = center_y = 0;
    nbIter = 30;
    //compute the solution at initial scale
    initGradients();
    //initBlurConstraints();//we don't reblur to be faster
    initColorConstraints(false);
    iterJacobiColor();
    //iterJacobiBlur();
    //reblur();
    //put back to the current state
    zoom = z;
    center_x = cx;
    center_y = cy;
    nbIter = nbI;

    //copy in the reference buffer
    glViewport(0,0,w[0],h[0]);
    glMatrixMode(GL_PROJECTION);glLoadIdentity();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_referenceBufferFBO);
    glColor3f(1.0,1.0,1.0);
    glActiveTexture(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    glActiveTexture(GL_TEXTURE0_ARB);
    glEnable( GL_TEXTURE_RECTANGLE_ARB );
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[0]);
    glBegin(GL_QUADS);
    glTexCoord2f(0,0);glVertex2f(-1,-1);
    glTexCoord2f(w[0],0);glVertex2f( 1,-1);
    glTexCoord2f(w[0],h[0]);glVertex2f( 1, 1);
    glTexCoord2f(0,h[0]);glVertex2f(-1, 1);
    glEnd();

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

void Solver::savePng(QString name)
{
    double z = zoom;
    double cx = center_x;
    double cy = center_y;
    int nbI = nbIter;
    bool activBlur = activateBlur;
    int bStep = blurStep;
    //put to the initial state
    zoom = 1;
    center_x = center_y = 0;
    nbIter = 50;
    activateBlur = true;
    blurStep = 1;
    //compute the solution at initial scale
    initGradients();
    initBlurConstraints();
    initColorConstraints(false);
    iterJacobiColor();
    iterJacobiBlur();
    reblur();
    //put back to the current state
    zoom = z;
    center_x = cx;
    center_y = cy;
    nbIter = nbI;
    activateBlur = activBlur;
    blurStep = bStep;
    initGradients();
    initBlurConstraints();
    initColorConstraints(true);

    //save the frame buffer
    float* pixels = new float [4*_w*_h];
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBuffers[0]);
    //glReadPixels(0,0,_w,_h,GL_RGBA,GL_FLOAT,pixels);

    QImage res = QImage( _w, _h, QImage::Format_ARGB32);
    glReadPixels( 0, 0, _w, _h, GL_RGBA, GL_UNSIGNED_BYTE, res.bits());
    res = res.rgbSwapped();
    res = res.mirrored(false,true);
    res.save(name,"PNG");

    delete[] pixels;

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

GLuint Solver::getRefId()//return the reference buffer id
{
    return _referenceBufferFBO;
}

//draw and blur the gradient map at first scale
void Solver::drawFirstScaleGradient()
{
    glActiveTexture(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    glActiveTexture(GL_TEXTURE0_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    
    //---- x-derivative ----//
    //init the buffers to gray
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_gxBufferFBOs[0]);
    glClearColor(0.5f,0.5f,0.5f,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    //zoom and pan
    glMatrixMode(GL_MODELVIEW) ;
    glScalef(zoom,zoom,0);
    glTranslatef(center_x,center_y,0.0);
    splines->drawNormalX();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    //---- y-derivative ----//
    //init the buffers to gray
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_gyBufferFBOs[0]);
    glClearColor(0.5f,0.5f,0.5f,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    //zoom and pan
    glMatrixMode(GL_MODELVIEW) ;
    glScalef(zoom,zoom,0);
    glTranslatef(center_x,center_y,0.0);
    splines->drawNormalY();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

void Solver::initGradients()
{
    //----- FIRST SCALE -----//
    glViewport(0,0,w[0],h[0]);
    glColor3f(1.0,1.0,1.0);

    //--- derivatives ---
    drawFirstScaleGradient();

    //--- divergence ---
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_divBufferFBOs[0]);
    glActiveTexture(GL_TEXTURE0_ARB);
    glEnable( GL_TEXTURE_RECTANGLE_ARB );
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gxBuffers[0]);
    glActiveTexture(GL_TEXTURE1_ARB);
    glEnable( GL_TEXTURE_RECTANGLE_ARB );
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gyBuffers[0]);
    divProgram.activate();
    glUniform1iARB(divProgram["gxMap"],0);
    glUniform1iARB(divProgram["gyMap"],1);
    glBegin(GL_QUADS);
    glVertex2f(-1,-1);
    glVertex2f( 1,-1);
    glVertex2f( 1, 1);
    glVertex2f(-1, 1);
    glEnd();
    divProgram.deactivate();

    //----- COARSER SCALES -----
    for(int d=1;d<deepness;d++)
    {
        glViewport(0,0,w[d],h[d]);

        //--- x-derivative ---
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_gxBufferFBOs[d]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glDisable( GL_TEXTURE_RECTANGLE_ARB );
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gxBuffers[d-1]);
        restrictProgram.activate();
        glUniform1iARB(restrictProgram["imgMap"],0);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        restrictProgram.deactivate();

        //--- y-derivative ---
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_gyBufferFBOs[d]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glDisable( GL_TEXTURE_RECTANGLE_ARB );
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gyBuffers[d-1]);
        restrictProgram.activate();
        glUniform1iARB(restrictProgram["imgMap"],0);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        restrictProgram.deactivate();

        //--- divergence ---
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_divBufferFBOs[d]);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gxBuffers[d]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gyBuffers[d]);
        divProgram.activate();
        glUniform1iARB(divProgram["gxMap"],0);
        glUniform1iARB(divProgram["gyMap"],1);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        divProgram.deactivate();
    }
	
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

void Solver::initColorConstraints(bool border)//draw the color constraints pyramid; border indicate if we have to add constraints on the image border
{
    //----- FIRST SCALE -----//
    glViewport(0,0,w[0],h[0]);
    
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_colorBufferFBOs[0]);
    //zoom and pan
    glMatrixMode(GL_MODELVIEW) ;
    glScalef(zoom,zoom,0);
    glTranslatef(center_x,center_y,0.0);
    //spline color constraints
    splines->drawColorConstraints(zoom);
    //image border
    if(border){
        double wz = w[0]/(2.0*zoom);
        double hz = h[0]/(2.0*zoom);
        double cx = 0.5*(-center_x+1)*w[0];
        double cy = 0.5*(-center_y+1)*h[0];
        Point P1(-1.0/zoom-center_x,-1.0/zoom-center_y+1.0/(zoom*h[0]));
        Point P2(1.0/zoom-center_x,-1.0/zoom-center_y+1.0/(zoom*h[0]));
        Point P3(1.0/zoom-center_x, 1.0/zoom-center_y);
        Point P4(-1.0/zoom-center_x, 1.0/zoom-center_y);

        //draw the textured border
        glActiveTexture(GL_TEXTURE1_ARB);
        glDisable( GL_TEXTURE_RECTANGLE_ARB );
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[0]);
        glColor4f(1.0,1.0,1.0,1.0);
        glBegin(GL_QUADS);
            glTexCoord2f(cx-wz,cy-hz);glVertex2f(P1.getX(),P1.getY());
            glTexCoord2f(cx+wz,cy-hz);glVertex2f(P2.getX(),P2.getY());
            glTexCoord2f(cx+wz,cy-hz);glVertex2f(P2.getX(),P2.getY()+1.0/(zoom*h[0]));
            glTexCoord2f(cx-wz,cy-hz);glVertex2f(P1.getX(),P1.getY()+1.0/(zoom*h[0]));

            glTexCoord2f(cx-wz,cy+hz);glVertex2f(P4.getX(),P4.getY());
            glTexCoord2f(cx+wz,cy+hz);glVertex2f(P3.getX(),P3.getY());
            glTexCoord2f(cx+wz,cy+hz);glVertex2f(P3.getX(),P3.getY()-1.0/(zoom*h[0]));
            glTexCoord2f(cx-wz,cy+hz);glVertex2f(P4.getX(),P4.getY()-1.0/(zoom*h[0]));

            glTexCoord2f(cx-wz,cy-hz);glVertex2f(P1.getX(),P1.getY());
            glTexCoord2f(cx-wz,cy+hz);glVertex2f(P4.getX(),P4.getY());
            glTexCoord2f(cx-wz,cy+hz);glVertex2f(P4.getX()+2.0/(zoom*w[0]),P4.getY());
            glTexCoord2f(cx-wz,cy-hz);glVertex2f(P1.getX()+2.0/(zoom*w[0]),P1.getY());

            glTexCoord2f(cx+wz,cy-hz);glVertex2f(P2.getX(),P2.getY());
            glTexCoord2f(cx+wz,cy+hz);glVertex2f(P3.getX(),P3.getY());
            glTexCoord2f(cx+wz,cy+hz);glVertex2f(P3.getX()-1.0/(zoom*w[0]),P3.getY());
            glTexCoord2f(cx+wz,cy-hz);glVertex2f(P2.getX()-1.0/(zoom*w[0]),P2.getY());
        glEnd();
    }

    //----- COARSER SCALES -----
    glColor3f(1.0,1.0,1.0);
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();
    for(int d=1;d<deepness;d++)
    {
        glViewport(0,0,w[d],h[d]);

        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_colorBufferFBOs[d]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glDisable( GL_TEXTURE_RECTANGLE_ARB );
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_colorBuffers[d-1]);
	restrictColorProgram.activate();
        glUniform1iARB(restrictColorProgram["imgMap"],0);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        restrictColorProgram.deactivate();
    }
	
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

void Solver::initBlurConstraints()
{
    glViewport(0,0,w[0],h[0]);
    
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurAmountBufferFBOs[0]);
    glClearColor(0.0f,0.0f,0.0f,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    glActiveTexture(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    glActiveTexture(GL_TEXTURE0_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );

    //zoom and pan
    glMatrixMode(GL_MODELVIEW) ;
    glScalef(zoom,zoom,0);
    glTranslatef(center_x,center_y,0.0);
    splines->drawBlurAmount();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    //----- COARSER SCALES -----
    glColor3f(1.0,1.0,1.0);
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();
    for(int d=1;d<deepness;d++)
    {
        glViewport(0,0,w[d],h[d]);

        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurAmountBufferFBOs[d]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glDisable( GL_TEXTURE_RECTANGLE_ARB );
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurAmountBuffers[d-1]);
	restrictColorProgram.activate();
        glUniform1iARB(restrictColorProgram["imgMap"],0);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        restrictColorProgram.deactivate();
    }

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

//draw the blur constraints at scale d
void Solver::drawBlurConstraints(int d)
{
    glEnable(GL_ALPHA_TEST);
    glAlphaFunc(GL_GREATER, 0.1f);
    
    glColor3f(1.0,1.0,1.0);
    glActiveTexture(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    glActiveTexture(GL_TEXTURE0_ARB);
    glEnable( GL_TEXTURE_RECTANGLE_ARB );
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurAmountBuffers[d]);
    glBegin(GL_QUADS);
    glTexCoord2f(0,0);glVertex2f(-1,-1);
    glTexCoord2f(w[d],0);glVertex2f( 1,-1);
    glTexCoord2f(w[d],h[d]);glVertex2f( 1, 1);
    glTexCoord2f(0,h[d]);glVertex2f(-1, 1);
    glEnd();
    
    glDisable(GL_ALPHA_TEST);
}

//draw the color constraints at scale d
void Solver::drawColorConstraints(int d)
{
    glEnable(GL_ALPHA_TEST);
    glAlphaFunc(GL_GREATER, 0.1f);
    
    glColor3f(1.0,1.0,1.0);
    glActiveTexture(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    glActiveTexture(GL_TEXTURE0_ARB);
    glEnable( GL_TEXTURE_RECTANGLE_ARB );
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_colorBuffers[d]);
    glBegin(GL_QUADS);
    glTexCoord2f(0,0);glVertex2f(-1,-1);
    glTexCoord2f(w[d],0);glVertex2f( 1,-1);
    glTexCoord2f(w[d],h[d]);glVertex2f( 1, 1);
    glTexCoord2f(0,h[d]);glVertex2f(-1, 1);
    glEnd();
    
    glDisable(GL_ALPHA_TEST);
}

//reblur the color image according to the blur map
void Solver::reblur()
{
    if(!activateBlur)
        return;

    float sigma = MAX_BLUR;
    int n=0;
    while(sigma>=0){
        n++;
        //first pass : result->temp
	sigma=sqrt(sigma*sigma-1);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[0]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[0]);
        anisotropicDiffusionProgram.activate();
        glUniform1iARB(anisotropicDiffusionProgram["imgMap"],0);
        glUniform1iARB(anisotropicDiffusionProgram["blurMap"],1);
        glUniform1fARB(anisotropicDiffusionProgram["blur"],sigma/(float)MAX_BLUR);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        anisotropicDiffusionProgram.deactivate();
    
        //second pass : temp->result
	//sigma=sqrt(sigma*sigma-1);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBufferFBOs[0]);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_tempBuffer);
        glActiveTexture(GL_TEXTURE1_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[0]);
        anisotropicDiffusionProgram.activate();
        glUniform1iARB(anisotropicDiffusionProgram["imgMap"],0);
        glUniform1iARB(anisotropicDiffusionProgram["blurMap"],1);
        glUniform1fARB(anisotropicDiffusionProgram["blur"],sigma/(float)MAX_BLUR);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        anisotropicDiffusionProgram.deactivate();
    }

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

//Poisson - Jacobi iterations on each level of the color pyramid
void Solver::iterJacobiColor()
{
    glViewport(0,0,w[deepness-1],h[deepness-1]);
    glMatrixMode(GL_PROJECTION);glLoadIdentity();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    //---- First scale (coarse) ----//
    //init the result to gray
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBufferFBOs[deepness-1]);
    glClearColor(1.0,1.0,1.0,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0,1.0,1.0);
    //draw the color constraints
    drawColorConstraints(deepness-1);

    //relaxation
    for(int i=0;i<nbIter*deepness;i++){
        //first pass : result->temp
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempJacobiBufferFBOs[deepness-1]);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[deepness-1]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_divBuffers[deepness-1]);
        jacobiPoissonProgram.activate();
        glUniform1iARB(jacobiPoissonProgram["imgMap"],0);
        glUniform1iARB(jacobiPoissonProgram["divMap"],1);
        glUniform1fARB(jacobiPoissonProgram["scale"],1);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        jacobiPoissonProgram.deactivate();
        //draw the color constraints
        drawColorConstraints(deepness-1);

        //second pass : temp->result
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBufferFBOs[deepness-1]);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_tempJacobiBuffers[deepness-1]);
        glActiveTexture(GL_TEXTURE1_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_divBuffers[deepness-1]);
        jacobiPoissonProgram.activate();
        glUniform1iARB(jacobiPoissonProgram["imgMap"],0);
        glUniform1iARB(jacobiPoissonProgram["divMap"],1);
        glUniform1fARB(jacobiPoissonProgram["scale"],1);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        jacobiPoissonProgram.deactivate();
        //draw the color constraints
        drawColorConstraints(deepness-1);
    }

    float scale;//scale corresponds to the scaling factor when accessing to
    //the result of a coarser level in the multigrid

    //---- propagation in the multigrid, with relaxation ----//
    for(int d=deepness-2;d>=0;d--)
    {
        glViewport(0,0,w[d],h[d]);
        glMatrixMode(GL_PROJECTION);glLoadIdentity();
        glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

        for(int i=0;i<nbIter*(d+1);i++){
            //first pass : result->temp
            glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempJacobiBufferFBOs[d]);
            glActiveTexture(GL_TEXTURE0_ARB);
            glEnable( GL_TEXTURE_RECTANGLE_ARB );
            if(i==0){
                glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[d+1]);
                scale=0.5f;
            }
            else{
                glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[d]);
                scale=1;
            }
            glActiveTexture(GL_TEXTURE1_ARB);
            glEnable( GL_TEXTURE_RECTANGLE_ARB );
            glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_divBuffers[d]);
            jacobiPoissonProgram.activate();
            glUniform1iARB(jacobiPoissonProgram["imgMap"],0);
            glUniform1iARB(jacobiPoissonProgram["divMap"],1);
            glUniform1fARB(jacobiPoissonProgram["scale"],scale);
            glBegin(GL_QUADS);
            glVertex2f(-1,-1);
            glVertex2f( 1,-1);
            glVertex2f( 1, 1);
            glVertex2f(-1, 1);
            glEnd();
            jacobiPoissonProgram.deactivate();
            //draw the color constraints
            drawColorConstraints(d);
    
            //second pass : temp->result
            glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBufferFBOs[d]);
            glActiveTexture(GL_TEXTURE0_ARB);
            glEnable( GL_TEXTURE_RECTANGLE_ARB );
            glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_tempJacobiBuffers[d]);
            glActiveTexture(GL_TEXTURE1_ARB);
            glEnable( GL_TEXTURE_RECTANGLE_ARB );
            glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_divBuffers[d]);
            jacobiPoissonProgram.activate();
            glUniform1iARB(jacobiPoissonProgram["imgMap"],0);
            glUniform1iARB(jacobiPoissonProgram["divMap"],1);
            glUniform1fARB(jacobiPoissonProgram["scale"],1);
            glBegin(GL_QUADS);
            glVertex2f(-1,-1);
            glVertex2f( 1,-1);
            glVertex2f( 1, 1);
            glVertex2f(-1, 1);
            glEnd();
            jacobiPoissonProgram.deactivate();
            //draw the color constraints
            drawColorConstraints(d);
        }
    }
	
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

//Laplace - Jacobi iterations on each level of the blur pyramid
void Solver::iterJacobiBlur()
{
    glViewport(0,0,w[deepness-1],h[deepness-1]);
    glMatrixMode(GL_PROJECTION);glLoadIdentity();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    //---- First scale (coarse) ----//
    //init the result to black
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurBufferFBOs[deepness-1]);
    glClearColor(0.0f,0.0f,0.0f,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0,1.0,1.0);
    //draw the blur constraints
    drawBlurConstraints(deepness-1);

    //relaxation
    for(int i=0;i<nbIter*deepness;i++){
        //first pass : result->temp
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempJacobiBufferFBOs[deepness-1]);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[deepness-1]);
        jacobiLaplaceProgram.activate();
        glUniform1iARB(jacobiLaplaceProgram["imgMap"],0);
        glUniform1fARB(jacobiLaplaceProgram["scale"],1);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        jacobiLaplaceProgram.deactivate();
        //draw the color constraints
        drawBlurConstraints(deepness-1);

        //second pass : temp->result
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurBufferFBOs[deepness-1]);
        glActiveTexture(GL_TEXTURE0_ARB);
        glEnable( GL_TEXTURE_RECTANGLE_ARB );
        glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_tempJacobiBuffers[deepness-1]);
        jacobiLaplaceProgram.activate();
        glUniform1iARB(jacobiLaplaceProgram["imgMap"],0);
        glUniform1fARB(jacobiLaplaceProgram["scale"],1);
        glBegin(GL_QUADS);
        glVertex2f(-1,-1);
        glVertex2f( 1,-1);
        glVertex2f( 1, 1);
        glVertex2f(-1, 1);
        glEnd();
        jacobiLaplaceProgram.deactivate();
        //draw the color constraints
        drawBlurConstraints(deepness-1);
    }

    float scale;//scale corresponds to the scaling factor when accessing to
    //the result of a coarser level in the multigrid

    //---- propagation in the multigrid, with relaxation ----//
    for(int d=deepness-2;d>=0;d--)
    {
        glViewport(0,0,w[d],h[d]);
        glMatrixMode(GL_PROJECTION);glLoadIdentity();
        glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

        for(int i=0;i<nbIter*(d+1);i++){
            //first pass : result->temp
            glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempJacobiBufferFBOs[d]);
            glActiveTexture(GL_TEXTURE0_ARB);
            glEnable( GL_TEXTURE_RECTANGLE_ARB );
            if(i==0){
                glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[d+1]);
                scale=0.5f;
            }
            else{
                glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[d]);
                scale=1;
            }
            jacobiLaplaceProgram.activate();
            glUniform1iARB(jacobiLaplaceProgram["imgMap"],0);
            glUniform1fARB(jacobiLaplaceProgram["scale"],scale);
            glBegin(GL_QUADS);
            glVertex2f(-1,-1);
            glVertex2f( 1,-1);
            glVertex2f( 1, 1);
            glVertex2f(-1, 1);
            glEnd();
            jacobiLaplaceProgram.deactivate();
            //draw the color constraints
            drawBlurConstraints(d);
    
            //second pass : temp->result
            glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_blurBufferFBOs[d]);
            glActiveTexture(GL_TEXTURE0_ARB);
            glEnable( GL_TEXTURE_RECTANGLE_ARB );
            glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_tempJacobiBuffers[d]);
            jacobiLaplaceProgram.activate();
            glUniform1iARB(jacobiLaplaceProgram["imgMap"],0);
            glUniform1fARB(jacobiLaplaceProgram["scale"],1);
            glBegin(GL_QUADS);
            glVertex2f(-1,-1);
            glVertex2f( 1,-1);
            glVertex2f( 1, 1);
            glVertex2f(-1, 1);
            glEnd();
            jacobiLaplaceProgram.deactivate();
            //draw the color constraints
            drawBlurConstraints(d);
        }
    }
	
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);
}

void Solver::fastUpdate()
{
    updateMultigrid();
    iterJacobiColor();
}

void Solver::slowUpdate()
{
    updateReference();
    updateMultigrid();
    iterJacobiColor();
    iterJacobiBlur();
    reblur();
}

void Solver::draw(int width, int height, DisplayMode display_mode)
{    
    glViewport(0,0,width,height);
    glMatrixMode(GL_PROJECTION);glLoadIdentity();
    glMatrixMode(GL_MODELVIEW) ;glLoadIdentity();

    glClearColor(1.0,1.0,1.0,1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0,1.0,1.0);

    if(display_mode==displayNOTHING)
        return;

    if(display_mode==displayCOLOR_CONSTRAINTS || displayBLUR_CONSTRAINTS){
        glEnable(GL_ALPHA_TEST);
        glAlphaFunc(GL_GREATER, 0.1f);
    }

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,0);
    glColor3f(1.0,1.0,1.0);
    glActiveTexture(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_RECTANGLE_ARB );
    glActiveTexture(GL_TEXTURE0_ARB);
    glEnable( GL_TEXTURE_RECTANGLE_ARB );
    switch(display_mode){
        case displayRESULT: glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_resultBuffers[deepness_debug]);break;
        case displayCOLOR_CONSTRAINTS: glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_colorBuffers[deepness_debug]);break;
        case displayBLUR_CONSTRAINTS: glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurAmountBuffers[deepness_debug]);break;
        case displayBLUR_MAP: glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_blurBuffers[deepness_debug]);break;
        case displayGX: glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gxBuffers[deepness_debug]);break;
        case displayGY: glBindTexture(GL_TEXTURE_RECTANGLE_ARB,_gyBuffers[deepness_debug]);break;
    }
    glBegin(GL_QUADS);
    glTexCoord2f(0,0);glVertex2f(-1,-1);
    glTexCoord2f(w[deepness_debug],0);glVertex2f( 1,-1);
    glTexCoord2f(w[deepness_debug],h[deepness_debug]);glVertex2f( 1, 1);
    glTexCoord2f(0,h[deepness_debug]);glVertex2f(-1, 1);
    glEnd();

    if(display_mode==displayCOLOR_CONSTRAINTS || displayBLUR_CONSTRAINTS){
        glDisable(GL_ALPHA_TEST);
    }
}

//increase the number of iterations
void Solver::increaseIter()
{
    if(nbIter<20)
        nbIter++;
}

//decrease the number of iterations
void Solver::decreaseIter()
{
    if(nbIter>1)
        nbIter--;
}

//set the number of iterations
void Solver::setIter(int i)
{
    nbIter=i;
}

//increase the level used for display (debug)
void Solver::increaseDeepnessDebug()
{
    if(deepness_debug<deepness-1)
        deepness_debug++;
}

//decrease the level used for display (debug)
void Solver::decreaseDeepnessDebug()
{
    if(deepness_debug>0)
        deepness_debug--;
}

void Solver::setZoom(double z)
{
    zoom = z;
}

void Solver::setCenterX(double cx)
{
    center_x = cx;
}

void Solver::setCenterY(double cy)
{
    center_y = cy;
}

void Solver::setActivateBlur(bool activate)
{
    activateBlur = activate;
}

QColor Solver::getPixel(double x, double y)
{
    float color[4];
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_resultBuffers[0]);
    glReadPixels((GLint)(x*w[0]),(GLint)(y*h[0]),1,1,GL_RGBA,GL_FLOAT,color);
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,_tempBufferFBO);

    return QColor((int)(color[0]*255), (int)(color[1]*255), (int)(color[2]*255));
}






